@@ -33,9 +33,9 @@ import pathlib
 import posixpath
 import re
 import shutil
-import sys
 import tarfile
 from abc import ABC
+from email.headerregistry import Address
 from email.message import EmailMessage
 from functools import partial
 from io import StringIO
@@ -44,6 +44,7 @@ from zipfile import ZipFile
 
 # 3rd party
 import click
+import toml
 from consolekit.terminal_colours import Fore, resolve_color_default
 from domdf_python_tools.paths import PathPlus, sort_paths, traverse_to_file
 from domdf_python_tools.stringlist import StringList
@@ -51,6 +52,7 @@ from domdf_python_tools.typing import PathLike
 from domdf_python_tools.words import word_join
 from first import first
 from shippinglabel.checksum import get_record_entry
+from shippinglabel.requirements import ComparableRequirement, combine_requirements
 
 # this package
 from whey.config import load_toml
@@ -148,11 +150,20 @@ class AbstractBuilder(ABC):
 
 		pkgdir = self.project_dir / self.config["package"]
 
+		if not pkgdir.is_dir():
+			raise FileNotFoundError(f"Package directory '{self.config['package']}' not found.")
+
+		found_file = False
+
 		for py_pattern in {"**/*.py", "**/*.pyi", "**/*.pyx", "**/py.typed"}:
 			for py_file in pkgdir.rglob(py_pattern):
 				if "__pycache__" not in py_file.parts:
+					found_file = True
 					yield py_file
 
+		if not found_file:
+			raise FileNotFoundError(f"No Python source files found in {pkgdir}")
+
 	def copy_source(self) -> None:
 		"""
 		Copy source files into the build directory.
@@ -310,29 +321,33 @@ class AbstractBuilder(ABC):
 		maintainer_email = []
 
 		for entry in self.config["authors"]:
-			if "name" in entry and "email" in entry:
-				author_email.append("{name} <{email}>".format_map(entry))
-			elif "email" in entry:
+			if entry["name"] and entry["email"]:
+				address = Address(entry["name"], addr_spec=entry["email"])
+				author_email.append(str(address))
+			elif entry["email"]:
 				author_email.append(entry["email"])
-			elif "name" in entry:
+			elif entry["name"]:
 				author.append(entry["name"])
 
 		for entry in self.config["maintainers"]:
-			if "name" in entry and "email" in entry:
+			if entry["name"] and entry["email"]:
 				maintainer_email.append("{name} <{email}>".format_map(entry))
-			elif "email" in entry:
+			elif entry["email"]:
 				maintainer_email.append(entry["email"])
-			elif "name" in entry:
+			elif entry["name"]:
 				maintainer.append(entry["name"])
 
-		if author:
-			metadata["Author"] = word_join(author)
+		# TODO: I'm not quite sure how PEP621 expects a name for one author and the email for another to be handled.
+
 		if author_email:
 			metadata["Author-email"] = ", ".join(author_email)
-		if maintainer:
-			metadata["Author"] = word_join(maintainer)
+		elif author:
+			metadata["Author"] = word_join(author)
+
 		if maintainer_email:
 			metadata["Author-email"] = ", ".join(maintainer_email)
+		elif maintainer:
+			metadata["Author"] = word_join(maintainer)
 
 		if self.config["license-key"] is not None:
 			metadata["License"] = self.config["license-key"]
@@ -352,7 +367,8 @@ class AbstractBuilder(ABC):
 		for classifier in self.config["classifiers"]:
 			metadata["Classifier"] = classifier
 
-		metadata["Requires-Python"] = str(self.config["requires-python"])
+		if self.config["requires-python"]:
+			metadata["Requires-Python"] = str(self.config["requires-python"])
 
 		for requirement in self.config["dependencies"]:
 			metadata["Requires-Dist"] = str(requirement)
@@ -415,6 +431,22 @@ class SDistBuilder(AbstractBuilder):
 		self._echo(Fore.GREEN(f"Source distribution created at {sdist_filename.resolve().as_posix()}"))
 		return os.path.basename(sdist_filename)
 
+	def write_pyproject_toml(self):
+		"""
+		Write the ``pyproject.toml`` file.
+		"""
+
+		# Copy pyproject.toml
+		pp_toml = toml.loads((self.project_dir / "pyproject.toml").read_text())
+		pp_toml.setdefault("build-system", {})
+		current_requires = map(ComparableRequirement, pp_toml["build-system"].get("requires", ()))
+		new_requirements = combine_requirements(ComparableRequirement("whey"), *current_requires)
+		pp_toml["build-system"]["requires"] = list(map(str, sorted(new_requirements)))
+		pp_toml["build-system"]["build-backend"] = "whey"
+		(self.build_dir / "pyproject.toml").write_clean(toml.dumps(pp_toml))
+		self.report_copied(self.project_dir / "pyproject.toml", self.build_dir / "pyproject.toml")
+		# TODO: perhaps make some of the dynamic fields static?
+
 	def build_sdist(self) -> str:
 		"""
 		Build the source distribution.
@@ -430,11 +462,9 @@ class SDistBuilder(AbstractBuilder):
 		self.copy_additional_files()
 		self.write_license(self.build_dir)
 
-		for filename in [
-				"pyproject.toml",
-				"requirements.txt",
-				]:
-			# TODO: ensure the build backend is set to whey (must be configurable to account for subclasses)
+		self.write_pyproject_toml()
+
+		for filename in ["requirements.txt"]:
 			source = self.project_dir / filename
 			if source.is_file():
 				dest = self.build_dir / filename
